@@ -14,7 +14,6 @@ package LaTeXML;
 use strict;
 use LaTeXML::Global;
 use LaTeXML::Error;
-use LaTeXML::Gullet;
 use LaTeXML::Stomach;
 use LaTeXML::Document;
 use LaTeXML::Model;
@@ -26,23 +25,27 @@ our @ISA = (qw(LaTeXML::Object));
 #use LaTeXML::Document;
 
 use vars qw($VERSION);
-$VERSION = "0.3.1";
+$VERSION = "0.5.9";
 
-#**********************************************************************
-# What a Mess of Globals!
 #**********************************************************************
 
 sub new {
   my($class,%options)=@_;
-  my $state     = LaTeXML::State->new(catcodes=>'standard');
-  $state->assignValue(VERBOSITY => (defined $options{verbosity} ? $options{verbosity} : 0), 'global');
-  $state->assignValue(STRICT    => (defined $options{strict}   ? $options{strict}     : 0), 'global');
+  my $state     = LaTeXML::State->new(catcodes=>'standard',
+				      stomach=>LaTeXML::Stomach->new(),
+				      model  => $options{model} || LaTeXML::Model->new());
+  $state->assignValue(VERBOSITY=>(defined $options{verbosity} ? $options{verbosity} : 0),
+		      'global');
+  $state->assignValue(STRICT   =>(defined $options{strict}   ? $options{strict}     : 0),
+		      'global');
   $state->assignValue(INCLUDE_COMMENTS=>(defined $options{includeComments} ? $options{includeComments} : 1),
-		     'global');
+		      'global');
+  $state->assignValue(DOCUMENTID=>(defined $options{documentid} ? $options{documentid} : ''),
+		      'global');
   $state->assignValue(SEARCHPATHS=> [ @{$options{searchpaths} || []} ],'global');
   bless {state   => $state, 
-	 model   => $options{model} || LaTeXML::Model->new(),
 	 nomathparse=>$options{nomathparse}||0,
+	 preload=>$options{preload},
 	}, $class; }
 
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -66,6 +69,11 @@ sub convertString {
   return unless $digested;
   $self->convertDocument($digested); }
 
+
+sub getStatusMessage {
+  my($self)=@_;
+  $$self{state}->getStatusMessage; }
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 # Mid-level API.
 
@@ -73,71 +81,73 @@ sub digestFile {
   my($self,$file)=@_;
   $file =~ s/\.tex$//;
   local $STATE    = $$self{state};
-  local $GULLET   = LaTeXML::Gullet->new();
-  local $STOMACH  = LaTeXML::Stomach->new(); # The current Stomach;
-  local $MODEL    = $$self{model};   # The document model.
+  my $stomach  = $STATE->getStomach; # The current Stomach;
+  my $gullet   = $stomach->getGullet;
   # And, set fancy error handler for ANY die!
   local $SIG{__DIE__} = sub { LaTeXML::Error::Fatal(join('',@_)); };
+  local $SIG{INT} = sub { LaTeXML::Error::Fatal(join('',@_)); }; # ??
 
-  NoteProgress("\n(Digesting $file...");
-  $STOMACH->initialize;
-  map($GULLET->input($_,['ltxml','latexml']), 'TeX', @{$$self{preload} || []} );
+  NoteBegin("Digesting $file");
+  $stomach->initialize;
+  map($gullet->input($_,['ltxml']), 'TeX', @{$$self{preload} || []} );
 
-  my $pathname = pathname_find($file,types=>['tex']);
+  my $pathname = pathname_find($file,types=>['tex','']);
   Fatal("Cannot find TeX file $file") unless $pathname;
   my($dir,$name,$ext)=pathname_split($pathname);
   $STATE->pushValue(SEARCHPATHS=>$dir);
   $STATE->installDefinition(LaTeXML::Expandable->new(T_CS('\jobname'),undef,Tokens(Explode($name))));
-  $GULLET->input($pathname);
-  my $list = LaTeXML::List->new($STOMACH->digestNextBody);
+  $gullet->input($pathname);
+  my $list = LaTeXML::List->new($stomach->digestNextBody);
   if(my $env = $STATE->lookupValue('current_environment')){
     Error("Input ended while environment $env was open"); } 
-  $GULLET->flush;
-  NoteProgress(")");
+  $gullet->flush;
+  NoteEnd("Digesting $file");
   $list; }
 
 sub digestString {
   my($self,$string)=@_;
   local $STATE    = $$self{state};
-  local $GULLET   = LaTeXML::Gullet->new();
-  local $STOMACH  = LaTeXML::Stomach->new(); # The current Stomach;
-  local $MODEL    = $$self{model};   # The document model.
+  my $stomach  = $STATE->getStomach; # The current Stomach;
+  my $gullet   = $stomach->getGullet;
   # And, set fancy error handler for ANY die!
   local $SIG{__DIE__} = sub { LaTeXML::Error::Fatal(join('',@_)); };
+  local $SIG{INT} = sub { LaTeXML::Error::Fatal(join('',@_)); }; # ??
 
-  NoteProgress("\n(Digesting string...");
-  $STOMACH->initialize;
-  map($GULLET->input($_), 'TeX', @{$$self{preload} || []} );
-  $GULLET->openMouth(LaTeXML::Mouth->new($string),0);
+  NoteBegin("Digesting string");
+  $stomach->initialize;
+  map($gullet->input($_,['ltxml']), 'TeX', @{$$self{preload} || []} );
+  $gullet->openMouth(LaTeXML::Mouth->new($string),0);
   $STATE->installDefinition(LaTeXML::Expandable->new(T_CS('\jobname'),undef,Tokens(Explode("Unknown"))));
-  my $list = LaTeXML::List->new($STOMACH->digestNextBody); 
+  my $list = LaTeXML::List->new($stomach->digestNextBody); 
   if(my $env = $STATE->lookupValue('current_environment')){
     Error("Input ended while environment $env was open"); } 
-  $GULLET->flush;
-  NoteProgress(")");
+  $gullet->flush;
+  NoteEnd("Digesting string");
   $list; }
 
 sub convertDocument {
   my($self,$digested)=@_;
   local $STATE    = $$self{state};
-  local $GULLET   = LaTeXML::Gullet->new();
-  local $STOMACH  = LaTeXML::Stomach->new(); # The current Stomach;
-  local $MODEL    = $$self{model};   # The document model.
+  my $model    = $STATE->getModel;   # The document model.
 
   # And, set fancy error handler for ANY die!
   local $SIG{__DIE__} = sub { LaTeXML::Error::Fatal(join('',@_)); };
+  local $SIG{INT} = sub { LaTeXML::Error::Fatal(join('',@_)); }; # ??
 
   local $LaTeXML::DUAL_BRANCH= '';
-  my $document  = LaTeXML::Document->new();
+  my $document  = LaTeXML::Document->new($model);
 
-  NoteProgress("\n(Building");
-  $LaTeXML::MODEL->loadDocType(); # If needed?
+  NoteBegin("Building");
+  $model->loadDocType(); # If needed?
+  if(my $paths = $STATE->lookupValue('SEARCHPATHS')){
+    if($STATE->lookupValue('INCLUDE_COMMENTS')){
+      $document->insertPI('latexml',searchpaths=>join(',',@$paths)); }}
   $document->absorb($digested);
-  NoteProgress(")");
+  NoteEnd("Building");
 
-  NoteProgress("\n(Rewriting");
-  $MODEL->applyRewrites($document,$document->getDocument->documentElement);
-  NoteProgress(")");
+  NoteBegin("Rewriting");
+  $model->applyRewrites($document,$document->getDocument->documentElement);
+  NoteEnd("Rewriting");
 
   LaTeXML::MathParser->new()->parseMath($document) unless $$self{nomathparse};
   my $xml = $document->finalize();
@@ -182,15 +192,16 @@ But also see the convenient command line script L<latexml> which suffices for mo
 
 Creates a new LaTeXML object for transforming TeX files into XML. 
 
-   verbosity  : bigger makes it more verbose, smaller is quieter
-                0 is the default,
-   strict     : If true, undefined control sequences and 
-                invalid document constructs give fatal errors, 
-                instead of warnings.
-   includeComments : If false, comments will be excluded
-               from the result document.
-   preload    : an array of modules to preload
-   searchpath : an array of paths to be searched for Packages and style files.
+ verbosity  : Controls verbosity; higher is more verbose,
+              smaller is quieter. 0 is the default.
+ strict     : If true, undefined control sequences and 
+              invalid document constructs give fatal
+              errors, instead of warnings.
+ includeComments : If false, comments will be excluded
+              from the result document.
+ preload    : an array of modules to preload
+ searchpath : an array of paths to be searched for Packages
+              and style files.
 
 (these generally set config variables in the L<LaTeXML::State> object)
 
