@@ -14,9 +14,8 @@ package LaTeXML::Definition;
 use strict;
 use LaTeXML::Global;
 use Exporter;
-use LaTeXML::Object;
 use LaTeXML::Parameters;
-our @ISA = qw(LaTeXML::Object);
+use base qw(LaTeXML::Object);
 
 #**********************************************************************
 
@@ -29,14 +28,9 @@ sub isPrefix     { 0; }
 sub getLocator   { $_[0]->{locator}; }
 
 sub readArguments {
-  my($self)=@_;
+  my($self,$gullet)=@_;
   my $params = $$self{parameters};
-  ($params ? $params->readArguments : ()); }
-
-sub showInvocation {
-  my($self,@args)=@_;
-  my $params = $$self{parameters};
-  ($params ? $$self{cs}->untex.$params->untexArguments(@args) : $$self{cs}->untex); }
+  ($params ? $params->readArguments($gullet,$self) : ()); }
 
 sub getParameters { $_[0]->{parameters}; }
 #======================================================================
@@ -45,23 +39,23 @@ sub stringify {
   my($self)=@_;
   my $type = ref $self;
   $type =~ s/^LaTeXML:://;
-  $type.'['.($$self{alias}||$$self{cs}->getCSName).Stringify($$self{parameters}).']'; }
+  $type.'['.($$self{alias}||$$self{cs}->getCSName).' '.Stringify($$self{parameters}).']'; }
 
 sub toString {
   my($self)=@_;
-  $$self{cs}->toString.($$self{parameters} ||''); }
+  $$self{cs}->toString.' '.ToString($$self{parameters} ||''); }
 
 # Return the Tokens that would invoke the given definition with arguments.
 sub invocation {
   my($self,@args)=@_;
-  Tokens($$self{cs},$$self{parameters}->invocationArguments(@args)); }
+  ($$self{cs},$$self{parameters}->revertArguments(@args)); }
 
 #**********************************************************************
 # Expandable control sequences (& Macros);  Expanded in the Gullet.
 #**********************************************************************
 package LaTeXML::Expandable;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Definition);
+use base qw(LaTeXML::Definition);
 
 # Known traits:
 #    isConditional: whether this expandable is some form of \ifxxx
@@ -74,9 +68,9 @@ sub new {
     foreach my $t ($expansion->unlist){
       $level++ if $t->equals(T_BEGIN);
       $level-- if $t->equals(T_END); }
-    Fatal("Defining Macro ".Stringify($cs).": replacement has unbalanced {}: ".$expansion->untex) if $level;  }
+    Fatal("Defining Macro ".Stringify($cs).": replacement has unbalanced {}: ".ToString($expansion)) if $level;  }
   bless {cs=>$cs, parameters=>$parameters, expansion=>$expansion,
-	 locator=>"defined ".$GULLET->getLocator,
+	 locator=>"defined ".$STATE->getStomach->getGullet->getLocator,
 	 %traits}, $class; }
 
 sub isExpandable  { 1; }
@@ -84,12 +78,17 @@ sub isConditional { $_[0]->{isConditional}; }
 
 # Expand the expandable control sequence. This should be carried out by the Gullet.
 sub invoke {
-  my($self)=@_;
-  my @args = $self->readArguments;
+  my($self,$gullet)=@_;
+  if($self->isConditional){
+    $STATE->assignValue(current_if_level=>($STATE->lookupValue('current_if_level')||0)+1, 'global'); }
+  $self->doInvocation($gullet,$self->readArguments($gullet)); }
+
+sub doInvocation {
+  my($self,$gullet,@args)=@_;
   my $expansion = $$self{expansion};
   (ref $expansion eq 'CODE' 
-   ? &$expansion($self,@args)
-   : substituteTokens($expansion,@args)); }
+   ? &$expansion($gullet,@args)
+   : substituteTokens($expansion,map($_ && Tokens($_->revert),@args))); }
 
 sub substituteTokens {
   my($tokens,@args)=@_;
@@ -117,7 +116,7 @@ sub equals {
 
 package LaTeXML::Primitive;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Definition);
+use base qw(LaTeXML::Definition);
 
 # Known traits:
 #    isPrefix : whether this primitive is a TeX prefix, \global, etc.
@@ -126,30 +125,27 @@ sub new {
   # Could conceivably have $replacement being a List or Box?
   Fatal("Defining Primitive ".Stringify($cs)." but replacement is not CODE: $replacement.")
     unless ref $replacement eq 'CODE';
-#  $cs = $cs->untex if ref $cs;
   bless {cs=>$cs, parameters=>$parameters, replacement=>$replacement,
-	 locator=>"defined ".$GULLET->getLocator, %traits}, $class; }
+	 locator=>"defined ".$STATE->getStomach->getGullet->getLocator, %traits}, $class; }
 
 sub isPrefix      { $_[0]->{isPrefix}; }
 
 sub executeBeforeDigest {
-  my($self)=@_;
+  my($self,$stomach)=@_;
   my $pre = $$self{beforeDigest};
-  ($pre ? map(&$_($self), @$pre) : ()); }
+  ($pre ? map(&$_($stomach), @$pre) : ()); }
 
 sub executeAfterDigest {
-  my($self,@whatever)=@_;
+  my($self,$stomach,@whatever)=@_;
   my $post = $$self{afterDigest};
-  ($post ? map(&$_(@whatever), @$post) : ()); }
+  ($post ? map(&$_($stomach,@whatever), @$post) : ()); }
 
 # Digest the primitive; this should occur in the stomach.
 sub invoke {
-  my($self)=@_;
-  my @pre = $self->executeBeforeDigest;
-  my @args = $self->readArguments;
-  (@pre,
-   &{$$self{replacement}}($self, @args),
-   $self->executeAfterDigest($self,@args)); }
+  my($self,$stomach)=@_;
+  ($self->executeBeforeDigest($stomach),
+   &{$$self{replacement}}($stomach,$self->readArguments($stomach->getGullet)),
+   $self->executeAfterDigest($stomach)); }
 
 sub equals {
   my($self,$other)=@_;
@@ -159,11 +155,12 @@ sub equals {
 
 #**********************************************************************
 # A `Generalized' register;
-# includes the normal ones, as well as registers and 
-# eventually things like catcode, 
+# includes the normal ones, as well as paramters,
+# along with tables like catcode.
+
 package LaTeXML::Register;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Primitive);
+use base qw(LaTeXML::Primitive);
 
 # Known Traits:
 #    beforeDigest, afterDigest : code for before/after digestion daemons
@@ -172,7 +169,7 @@ sub new {
   my($class,$cs,$parameters,$type,$getter,$setter ,%traits)=@_;
   bless {cs=>$cs, parameters=>$parameters,
 	 registerType=>$type, getter => $getter, setter => $setter,
-	 locator=>"defined ".$GULLET->getLocator, %traits}, $class; }
+	 locator=>"defined ".$STATE->getStomach->getGullet->getLocator, %traits}, $class; }
 
 sub isPrefix    { 0; }
 sub isRegister { $_[0]->{registerType}; }
@@ -189,13 +186,36 @@ sub setValue {
 
 # No before/after daemons ???
 sub invoke {
-  my($self)=@_;
-  my @args = $self->readArguments;
-  $GULLET->readKeyword('=');	# Ignore 
-  my $value = $GULLET->readValue($self->isRegister);
+  my($self,$stomach)=@_;
+  my $gullet=$stomach->getGullet;
+  my @args = $self->readArguments($gullet);
+  $gullet->readKeyword('=');	# Ignore 
+  my $value = $gullet->readValue($self->isRegister);
   $self->setValue($value,@args);
   return; }
 
+#**********************************************************************
+# A CharDef is a specialized register;
+# You can't assign it; when you invoke the control sequence, it returns
+# the result of evaluating the character (more like a regular primitive).
+
+package LaTeXML::CharDef;
+use LaTeXML::Global;
+use base qw(LaTeXML::Register);
+
+sub new {
+  my($class,$cs,$value,%traits)=@_;
+  bless {cs=>$cs, parameters=>undef,
+	 value=>$value, char=>T_OTHER(chr($value->valueOf)),
+	 registerType=>'Number', readonly=>1,
+	 locator=>"defined ".$STATE->getStomach->getGullet->getLocator, %traits}, $class; }
+
+sub valueOf  { $_[0]->{value}; }
+sub setValue { Error("Cannot assign to chardef ".$_[0]->getCSName); return; }
+sub invoke   { 
+  my($self,$stomach)=@_;
+  $stomach->invokeToken($$self{char}); }
+
 #**********************************************************************
 # Constructor control sequences.  
 # They are first converted to a Whatsit in the Stomach, and that Whatsit's
@@ -206,11 +226,11 @@ sub invoke {
 package LaTeXML::Constructor;
 use strict;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Primitive);
+use base qw(LaTeXML::Primitive);
 
 # Known traits:
 #    beforeDigest, afterDigest : code for before/after digestion daemons
-#    untex : pattern for reverting to TeX form
+#    reversion : CODE or TOKENS for reverting to TeX form
 #    captureBody : whether to capture the following List as a `body` 
 #        (for environments, math modes)
 #    properties : a hash of default values for properties to store in the Whatsit.
@@ -219,58 +239,82 @@ sub new {
   Fatal("Defining Constructor ".Stringify($cs)." but replacement is not a string or CODE: $replacement")
     unless (defined $replacement) && (!(ref $replacement) || (ref $replacement eq 'CODE'));
   bless {cs=>$cs, parameters=>$parameters, replacement=>$replacement,
-	 locator=>"defined ".$GULLET->getLocator, %traits,
+	 locator=>"defined ".$STATE->getStomach->getGullet->getLocator, %traits,
+#	 nargs =>(defined $traits{nargs} ? $traits{nargs}
+#		  : ($parameters ? scalar(grep(! $_->getNoValue, $parameters->getParameters))
+#		     : 0))}, $class; }
 	 nargs =>(defined $traits{nargs} ? $traits{nargs}
-		  : ($parameters ? scalar(grep(! $_->getNoValue, $parameters->getParameters))
-		     : 0))}, $class; }
+		  : ($parameters ? $parameters->getNumArgs : 0))}, $class; }
 
-sub getConstructor {
-  my($self)=@_;
-  my $replacement = $$self{replacement};
-  if(!ref $replacement){
-    $$self{replacement} = $replacement 
-      = LaTeXML::ConstructorCompiler::compileConstructor($replacement,$self->getCS,
-#							 ($$self{parameters} ? $$self{parameters}->getNArgs:0));}
-							 $$self{nargs});}
-  $replacement; }
-
-sub getUntexSpec { $_[0]->{untex}; }
+sub getReversionSpec { $_[0]->{reversion}; }
 sub getAlias     { $_[0]->{alias}; }
 
 # Digest the constructor; This should occur in the Stomach to create a Whatsit.
 # The whatsit which will be further processed to create the document.
 sub invoke {
-  my($self)=@_;
+  my($self,$stomach)=@_;
   # Call any `Before' code.
-  my @pre = $self->executeBeforeDigest;
+  my @pre = $self->executeBeforeDigest($stomach);
+
+  # Get some info before we process arguments...
+  my $font = $STATE->lookupValue('font');
+  my $ismath = $STATE->lookupValue('IN_MATH');
   # Parse AND digest the arguments to the Constructor
-  my @args = $self->readArguments;
-  if(my $params = $$self{parameters}){
-    @args = $params->digestArguments(@args); }
+  my $params = $$self{parameters};
+  my @args = ($params ? $params->readArgumentsAndDigest($stomach,$self) : ());
   @args = @args[0..$$self{nargs}-1];
-  my %props = %{$$self{properties} || {} };
+
+  # Compute any extra Whatsit properties (many end up as element attributes)
+  my $properties = $$self{properties};
+  my %props = (!defined $properties ? ()
+	       : (ref $properties eq 'CODE' ? &$properties($stomach,@args)
+		  : %$properties));
   foreach my $key (keys %props){
     my $value = $props{$key};
     if(ref $value eq 'CODE'){
-      $props{$key} = &$value($self); }
+      $props{$key} = &$value($stomach,@args); }
     elsif($value && ($value =~/^\#(\d)$/)){
       $props{$key} = $args[$1-1]->toString; }}
+  $props{font}    = $font   unless defined $props{font};
+  $props{locator} = $stomach->getGullet->getLocator unless defined $props{locator};
+  $props{isMath}  = $ismath unless defined $props{isMath};
+  $props{level}   = $stomach->getBoxingLevel;
+
+  # Now create the Whatsit, itself.
   my $whatsit = LaTeXML::Whatsit->new($self,[@args],%props);
-  my @post = $self->executeAfterDigest($whatsit,@args);
+
+  # Call any 'After' code.
+  my @post = $self->executeAfterDigest($stomach,$whatsit);
   if(my $id = $props{id}){
     $STATE->assignValue('xref:'.$id=>$whatsit,'global'); }
   if($$self{captureBody}){
-    $whatsit->setBody(@post,$STOMACH->digestNextBody); @post=(); }
+    $whatsit->setBody(@post,$stomach->digestNextBody); @post=(); }
   (@pre,$whatsit,@post); }
 
+sub doAbsorbtion {
+  my($self,$document,$whatsit)=@_;
+  # First, compile the constructor pattern, if needed.
+  my $replacement = $$self{replacement};
+  if(!ref $replacement){
+    $$self{replacement} = $replacement
+      = LaTeXML::ConstructorCompiler::compileConstructor($replacement,$self->getCS,$$self{nargs}); }
+  # Now do the absorbtion.
+  if(my $pre = $$self{beforeConstruct}){
+    map(&$_($document,$whatsit), @$pre); }
+  &{$replacement}($document,$whatsit->getArgs, $whatsit->getProperties); 
+  if(my $post = $$self{afterConstruct}){
+    map(&$_($document,$whatsit), @$post); }
+}
+
 #**********************************************************************
 package LaTeXML::ConstructorCompiler;
 use strict;
 use LaTeXML::Global;
-our $VALUE_RE = "(\\#)";
-our $COND_RE  = "\\?($VALUE_RE|IfMath)";
+
+our $VALUE_RE = "(\\#|\\&[\\w\\:]*\\()";
+our $COND_RE  = "\\?$VALUE_RE";
 our $QNAME_RE = "([\\w\\-_:]+)";
-our $TEXT_RE  = "(.[^\\#<\\?]*)";
+our $TEXT_RE  = "(.[^\\#<\\?\\)\\&\\,]*)";
 
 our $GEN=0;
 sub compileConstructor {
@@ -285,11 +329,13 @@ sub compileConstructor {
   my $body = translate_constructor($constructor,$floats);
   my $code =
     " sub $name {\n"
-    ."my(".join(', ','$document', map("\$arg$_",1..$nargs),'$prop').")=\@_;\n"
+    ."my(".join(', ','$document', map("\$arg$_",1..$nargs),'%prop').")=\@_;\n"
       .($floats ? "my \$savenode;\n" :'')
 	. $body
 	  . ($floats ? "\$document->setNode(\$savenode) if defined \$savenode;\n" : '')
 	    . "}\n" ;
+###print STDERR "Compilation of \"$constructor\" => \n$code\n";
+
   eval $code;
   Fatal("Compilation of \"$constructor\" => \n$code\nFailed; $@") if $@; 
   \&$name; }
@@ -312,6 +358,7 @@ sub translate_constructor {
     elsif(s|^\s*<$QNAME_RE||so){
       my($tag,$av) = ($1,translate_avpairs());
       if($float){
+#	$code .= "\$savenode=\$document->floatToElement('$tag') unless \$document->isOpenable('$tag');\n";
 	$code .= "\$savenode=\$document->floatToElement('$tag');\n";
 	$float = undef; }
       $code .= "\$document->openElement('$tag'".($av? ", $av" : '').");\n";
@@ -344,22 +391,37 @@ sub slashify {
 # parse a conditional in a constructor
 # Conditionals are of the form ?value(...)(...),
 # Return the translated condition, along with the strings for the if and else clauses.
-# It does NOT handled nested conditionals!!! (yet...)
+use Text::Balanced;
 sub parse_conditional {
   s/^\?//;			# Remove leading "?"
-  my $bool = (s/^IfMath// ? '$$prop{isMath}'
-	      : 'ToString('.translate_value().')' );
-  if(s/^\((.*?)\)(\((.*?)\))?//s){
-    ($bool,$1,$3); }
+  my $bool =  'ToString('.translate_value().')';
+##  if(s/^\((.*?)\)(\((.*?)\))?//s){
+##    ($bool,$1,$3); }
+  if(my $if = Text::Balanced::extract_bracketed($_,'()')){
+    $if =~ s/^\(//;    $if =~ s/\)$//;
+    my $else = Text::Balanced::extract_bracketed($_,'()');
+    $else =~ s/^\(// if $else;    $else =~ s/\)$// if $else;
+    ($bool,$if,$else); }
   else {
     Fatal("Unbalanced conditional in constructor template \"$_\""); }}
 
 # Parse a substitutable value from the constructor (in $_)
-# Recognizes the #1, %prop, possibly followed by {foo}, for KeyVals,
-# Future enhancements? array ref, &foo(xxx) for function calls, ...
+# Recognizes the #1, #prop, and also &function(args,...)
 sub translate_value {
   my $value;
-  if   (s/^\#(\d+)//     ){
+  if(s/^\&([\w\:]*)\(//){	# Recognize a function call, w/args
+    my $fcn = $1;
+    # Hack: If no explict package, assume name it must be accessible via Pool
+    $fcn = "LaTeXML::Package::Pool::$fcn" unless $fcn =~/:/;
+    my @args = ();
+    while(! /^\s*\)/){
+      if(/^\s*([\'\"])/){ push(@args,translate_string()); }
+      else              { push(@args,translate_value()); }
+      last unless s/^\s*\,\s*//; }
+    Error("Missing ')' in &$fcn(...) in constructor pattern for $LaTeXML::ConstructorCompiler::NAME")
+      unless s/\)//;
+    $value = "$fcn(".join(',',@args).")"; }
+  elsif(s/^\#(\d+)//     ){	# Recognize an explicit #1 for whatsit args
     my $n = $1;
     if(($n < 1) || ($n > $LaTeXML::ConstructorCompiler::NARGS)){
       Error("Illegal argument number $n in constructor for "
@@ -367,13 +429,8 @@ sub translate_value {
       $value = "\"Missing\""; }
     else {
       $value = "\$arg$n" }}
-  elsif(s/^\#([\w\-_]+)//){ $value = "\$\$prop{'$1'}"; }
+  elsif(s/^\#([\w\-_]+)//){ $value = "\$prop{'$1'}"; } # Recognize #prop for whatsit properties
   elsif(s/$TEXT_RE//so    ){ $value = "'".slashify($1)."'"; }
-  # &foo(...) ? Function (but not &foo; !!!)
-  # Maybe shouldn't give an error?????
-  if(s/^\{$QNAME_RE\}//o){				       # Then accept {key} modifier.
-    $value = "((defined $value)&& $value->getValue('$1'))"; }
-  # Array???
   $value; }
 
 # Parse a delimited string from the constructor (in $_), 
@@ -417,13 +474,9 @@ sub translate_avpairs {
       { local $_=$else; $code .= translate_avpairs() if $else; }
       $code .= "))";
       push(@avs,$code); }
-    elsif(s/^\#(\d+)//){ 	# Stand alone value better be a KeyVal
-      my $n = $1;
-      if(($n < 1) || ($n > $LaTeXML::ConstructorCompiler::NARGS)){
-	Error("Illegal argument number $n in constructor for "
-	      ."$LaTeXML::ConstructorCompiler::NAME which takes $LaTeXML::ConstructorCompiler::NARGS args");}
-      else {
-	push(@avs,"LaTeXML::ConstructorCompiler::keyvalAttributes(\$arg$n)"); }}
+    elsif(/^%$VALUE_RE/){	# Hash?  Assume the value can be turned into a hash!
+      s/^%//;			# Eat the "%" 
+      push(@avs,'%{'.translate_value().'}'); }
     elsif(s|^$QNAME_RE\s*=\s*||o){
       my ($key,$value) = ($1,translate_string());
       push(@avs,"'$key'=>$value"); } # if defined $value; }
@@ -431,14 +484,6 @@ sub translate_avpairs {
     s|^\s*||; }
   join(', ',@avs); }
 
-sub keyvalAttributes {
-  my($kv)=@_;
-  if(!defined $kv){ (); }
-  elsif(!(ref $kv) || !$kv->isa('LaTeXML::KeyVals')){
-    Error("Argument \"".Stringify($kv)."\" used as KeyVals in constructor pattern");
-    (); }
-  else {
-    map( ($_=>$kv->getValue($_)), $kv->getKeys); }}
 #**********************************************************************
 1;
 
@@ -448,18 +493,20 @@ __END__
 
 =head1 NAME
 
-C<LaTeXML::Definition>, C<LaTeXML::Expandable>, C<LaTeXML::Primitive>, 
-C<LaTeXML::Register>, C<LaTeXML::Constructor> -- Control sequence definitions.
+C<LaTeXML::Definition>  - Control sequence definitions,
+including specializations C<LaTeXML::Expandable>, C<LaTeXML::Primitive>, 
+C<LaTeXML::Register>, C<LaTeXML::Constructor>
 
 =head1 DESCRIPTION
 
 These represent the various executables corresponding to control sequences.
+See L<LaTeXML::Package> for the most convenient means of creating them.
 
 =over 4
 
 =item C<LaTeXML::Expandable>
 
- represents macros and other expandable control sequences like C<\if>, etc
+represents macros and other expandable control sequences like C<\if>, etc
 that are carried out in the Gullet during expansion. The results of invoking an
 C<LaTeXML::Expandable> should result in a list of C<LaTeXML::Token>s.
 
@@ -467,9 +514,9 @@ C<LaTeXML::Expandable> should result in a list of C<LaTeXML::Token>s.
 
 represents primitive control sequences that are primarily carried out
 for side effect during digestion in the L<LaTeXML::Stomach> and for changing
-the L<LaTeXML::State>.  The results of invoking a C<LaTeXML::Primitive>, if any, 
-should be a list of C<LaTeXML::Box>s, C<LaTeXML::List>'s
-or C<LaTeXML::Whatsit>s.
+the L<LaTeXML::State>.  The results of invoking a C<LaTeXML::Primitive>, if any,
+should be a list of digested items (C<LaTeXML::Box>, C<LaTeXML::List>
+or C<LaTeXML::Whatsit>).
 
 =item C<LaTeXML::Register>
 
@@ -493,11 +540,31 @@ More documentation needed, but see LaTeXML::Package for the main user access to
 
 =over 4
 
-=item C<< $defn->readArguments; >>
+=item C<< $token = $defn->getCS; >>
 
-Reads the arguments for this C<$defn> from the current C<$GULLET>,
+Returns the (main) token that is bound to this definition.
+
+=item C<< $string = $defn->getCSName; >>
+
+Returns the string form of the token bound to this definition,
+taking into account any alias for this definition.
+
+=item C<< $defn->readArguments($gullet); >>
+
+Reads the arguments for this C<$defn> from the C<$gullet>,
 returning a list of L<LaTeXML::Tokens>.
 
+=item C<< $parameters = $defn->getParameters; >>
+
+Return the C<LaTeXML::Parameters> object representing the formal parameters
+of the definition.
+
+=item C<< @tokens = $defn->invocation(@args); >>
+
+Return the tokens that would invoke the given definition with the
+provided arguments.  This is used to recreate the TeX code (or it's
+equivalent).
+
 =item C<< $defn->invoke; >>
 
 Invoke the action of the C<$defn>.  For expandable definitions, this is done in
@@ -512,13 +579,40 @@ that are executed before and after digestion.  These can be useful for changing
 
 =back
 
+=head2 More about Primitives
+
+Primitive definitions may have lists of subroutines, called C<beforeDigest> and C<afterDigest>,
+that are executed before (and before the arguments are read) and after digestion.
+These should either end with C<return;>, C<()>, or return a list of digested 
+objects (L<LaTeXML::Box> or similar) that will be contributed to the current list.
+
+=head2 More about Registers
+
+Registers generally store some value in the current C<LaTeXML::State>, but are not
+required to. Like TeX's registers, when they are digested, they expect an optional
+C<=>, and then a value of the appropriate type. Register definitions support these
+additional methods:
+
+=over 4
+
+=item C<< $value = $register->valueOf(@args); >>
+
+Return the value associated with the register, by invoking it's C<getter> function.
+The additional args are used by some registers
+to index into a set, such as the index to C<\count>.
+
+=item C<< $register->setValue($value,@args); >>
+
+Assign a value to the register, by invoking it's C<setter> function.
+
+=back
+
 =head2 More about Constructors
 
 A constructor has as it's C<replacement> either a subroutine, or a string pattern representing
 the XML fragment it should generate.  In the case of a string pattern, the pattern is
 compiled into a subroutine on first usage by the internal class C<LaTeXML::ConstructorCompiler>.
-See L<LaTeXML::Package> for a full description of the syntax.
-
+Like primitives, constructors may have C<beforeDigest> and C<afterDigest>.
 
 =head1 AUTHOR
 
