@@ -23,12 +23,15 @@ sub generateMessage {
   my($type,$message,$long,@extra)=@_;
   my @lines=("\n".$type.": ".$message);
   $long = 0 if $STATE->lookupValue('VERBOSITY') < -1;
-  $long = 1 if $STATE->lookupValue('VERBOSITY') > +1;
-  if(my @objects = objectStack( ($long ? undef : 1))){
+  $long ++  if $STATE->lookupValue('VERBOSITY') > +1;
+  my $nstack =  ($long > 1 ? undef : ($long ? 4 : 1));
+  if(my @objects = objectStack($nstack)){
     my $top = shift(@objects);
     push(@lines,"In ".trim(Stringify($top)).' '.Stringify(Locator($top)));
+    push(@objects,'...') if @objects && defined $nstack;
     push(@lines,join('',map(' <= '.trim(Stringify($_)),@objects))) if @objects; }
-  push(@lines,$GULLET->getLocator($long)) if $GULLET;
+  if(my $stomach = $STATE->getStomach){
+    push(@lines,$stomach->getGullet->getLocator($long)); }
   join("\n",grep($_,@lines, @extra)); }
 
 sub Locator {
@@ -50,6 +53,7 @@ our $MAXLEN=40;			# Or more?
 sub trim {
   my($string)=@_;
   substr($string,$MAXLEN-3) = "..." if(length($string) > $MAXLEN);
+  $string =~ s/\n/\x{240D}/gs;	# symbol for CR 
   $string; }
 
 sub caller_info {
@@ -151,11 +155,11 @@ sub line_in_file {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
-C<LaTeXML::Error> -- Error reporting code.
+C<LaTeXML::Error> - Internal Error reporting code.
 
 =head1 DESCRIPTION
 
@@ -169,15 +173,15 @@ No user serviceable parts inside.  No symbols are exported.
 
 =over 4
 
-=item C<< $string = LaTeXML::Error::generateMessage($type,$message,$long,@extra); >>
+=item C<< $string = LaTeXML::Error::generateMessage($typ,$msg,$long,@more); >>
 
 Constructs an error or warning message based on the current stack and
 the current location in the document.
-C<$type> is a short string characterizing the type of message, such as "Error".  
-C<$message> is the error message itself. If C<$long> is true, will generate a
+C<$typ> is a short string characterizing the type of message, such as "Error".  
+C<$msg> is the error message itself. If C<$long> is true, will generate a
 more verbose message; this also uses the VERBOSITY set in the C<$STATE>.
 Longer messages will show a trace of the objects invoked on the stack,
-C<@extra> are additional strings to include in the message.
+C<@more> are additional strings to include in the message.
 
 =item C<< $string = LaTeXML::Error::stacktrace; >>
 
