@@ -13,8 +13,7 @@
 package LaTeXML::Font;
 use strict;
 use LaTeXML::Global;
-use LaTeXML::Object;
-our @ISA = qw(LaTeXML::Object);
+use base qw(LaTeXML::Object);
 
 our $DEFFAMILY = 'serif';
 our $DEFSERIES = 'medium';
@@ -47,7 +46,6 @@ sub getSize   { $_[0]->[3]; }
 sub getColor  { $_[0]->[4]; }
 
 sub toString { "Font[".join(',',map($_ || '*', @{$_[0]}))."]"; }
-sub untex    { $_[0]->toString; }
 sub stringify{ $_[0]->toString; }
 
 sub equals {
@@ -83,7 +81,7 @@ sub merge {
   (ref $self)->new_internal($family,$series,$shape,$size,$color); }
 
 # Return a string representing the font relative to other.
-sub relativeTo {
+sub XXrelativeTo {
   my($self,$other)=@_;
   my($family,$series,$shape,$size,$color)=@$self;  
   my($ofamily,$oseries,$oshape,$osize,$ocolor)=@$other;
@@ -97,6 +95,23 @@ sub relativeTo {
 		   ($color  && (!$ocolor  || ($color  ne $ocolor))  ? $color : ''));
   join(' ',@diffs); }
 
+# Return a hash of the differences in font, size and color
+sub relativeTo {
+  my($self,$other)=@_;
+  my($family,$series,$shape,$size,$color)=@$self;  
+  my($ofamily,$oseries,$oshape,$osize,$ocolor)=@$other;
+  $family  = 'serif' if $family  && ($family eq 'math');
+  $ofamily = 'serif' if $ofamily && ($ofamily eq 'math');
+  my @diffs=(($family && (!$ofamily || ($family ne $ofamily)) ? $family : ''),
+	     ($series && (!$oseries || ($series ne $oseries)) ? $series : ''),
+	     ($shape  && (!$oshape  || ($shape  ne $oshape))  ? $shape : ''));
+  my $fdiff=join(' ',grep($_, @diffs)); 
+  my $sdiff=($size   && (!$osize   || ($size   ne $osize))   ? $size : '');
+  my $cdiff=($color  && (!$ocolor  || ($color  ne $ocolor))  ? $color : '');
+  ( ($fdiff ? (font=>$fdiff):()),
+    ($sdiff ? (size=>$sdiff):()),
+    ($cdiff ? (color=>$cdiff):()) ); }
+
 sub distance {
   my($self,$other)=@_;
   my($family,$series,$shape,$size,$color)=@$self;  
@@ -111,9 +126,11 @@ sub distance {
 
 # This matches fonts when both are converted to strings (toString),
 # such as when they are set as attributes.
-sub match_font {
+sub XXmatch_font {
   my($font1,$font2)=@_;
 #print STDERR "Match font \"".($font1 || 'none')."\" to \"".($font2||'none')."\"\n";
+return 1;
+
   return 0 unless $font1 && $font2;
   $font1 =~ /^Font\[(.*)\]$/;
   my @comp1  = split(',',$1);
@@ -125,11 +142,48 @@ sub match_font {
     return 0 if ($c1 ne '*') && ($c2 ne '*') && ($c1 ne $c2); }
   return 1; }
 
+
+our %FONT_REGEXP_CACHE=();
+
+sub match_font {
+  my($font1,$font2)=@_;
+  my $regexp = $FONT_REGEXP_CACHE{$font1};
+  if(!$regexp){
+    $font1 =~ /^Font\[(.*)\]$/;
+    my @comp  = split(',',$1);
+    my $re= '^Font\['
+      . join(',', map( ($_ eq '*' ? "[^,]+" : "\Q$_\E"), @comp))
+	.'\]$';
+    print STDERR "\nCreating re for \"$font1\" => $re\n";
+    $regexp = $FONT_REGEXP_CACHE{$font1} = qr/$re/; }
+  $font2 =~ /$regexp/; }
+
+
+sub font_match_xpaths {
+  my($font)=@_;
+  $font =~ /^Font\[(.*)\]$/;
+  my @comps  = split(',',$1);
+  my($frag,@frags) = ();
+  for(my $i=0; $i<=$#comps; $i++){
+    my $comp = $comps[$i];
+    if($comp eq '*'){
+      push(@frags,$frag) if $frag;
+      $frag = undef; }
+    else {
+      my $post = ($i == $#comps ? ']' : ',');
+      if($frag){
+	$frag .= $comp . $post; }
+      else {
+	$frag = ($i==0 ? 'Font[' : ',') . $comp . $post; }}}
+  push(@frags,$frag) if $frag;
+  join(' and ','@_font',
+       map("contains(\@_font,'$_')",@frags)); }
+
 #**********************************************************************
 package LaTeXML::MathFont;
 use strict;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Font);
+use base qw(LaTeXML::Font);
 
 our $DEFFAMILY = 'serif';
 our $DEFSERIES = 'medium';
@@ -225,7 +279,8 @@ __END__
 
 =head1 NAME
 
-C<LaTeXML::Font> and C<LaTeXML::MathFont> -- representation of fonts.
+C<LaTeXML::Font> - representation of fonts,
+along with the specialization C<LaTeXML::MathFont>.
 
 =head1 DESCRIPTION
 
@@ -238,17 +293,19 @@ C<LaTeXML::Font> and C<LaTeXML::MathFont> represent fonts
 
 The attributes are
 
-   family : serif, sansserif, typewriter, caligraphic, fraktur, script
-   series : medium, bold
-   shape  : upright, italic, slanted, smallcaps
-   size   : tiny, footnote, small, normal, large, Large, LARGE, huge, Huge
-   color  : any named color, default is black
+ family : serif, sansserif, typewriter, caligraphic,
+          fraktur, script
+ series : medium, bold
+ shape  : upright, italic, slanted, smallcaps
+ size   : tiny, footnote, small, normal, large,
+          Large, LARGE, huge, Huge
+ color  : any named color, default is black
 
 They are usually merged against the current font, attempting to mimic the,
 sometimes counter-intuitive, way that TeX does it,  particularly for math
 
-Additionally, C<LaTeXML::MathFont> supports C<$font->specialize($string);>, which
-computes a font reflecting how the specific C<$string> would be printed when
+Additionally, C<LaTeXML::MathFont> supports C<$font->specialize($string);> for
+computing a font reflecting how the specific C<$string> would be printed when
 C<$font> is active; This (attempts to) handle the curious ways that lower case
 greek often doesn't get a different font.  In particular, it recognizes the
 following classes of strings: single latin letter, single uppercase greek character,
