@@ -21,12 +21,13 @@
 #======================================================================
 package LaTeXML::Global;
 use strict;
-use Exporter;
 use XML::LibXML;
-our @ISA = qw(Exporter);
+use Time::HiRes;
+
+use base qw(Exporter);
 our @EXPORT = ( 
-	       # Global State accessors; These variables get bound by LaTeXML.pm
-	       qw( *STATE *GULLET *STOMACH *MODEL),
+	       # Global STATE; This gets bound by LaTeXML.pm
+	       qw( *STATE),
 	       # Catcode constants
 	       qw( CC_ESCAPE  CC_BEGIN  CC_END     CC_MATH
 		   CC_ALIGN   CC_EOL    CC_PARAM   CC_SUPER
@@ -37,11 +38,11 @@ our @EXPORT = (
 	       qw( &T_BEGIN &T_END &T_MATH &T_ALIGN &T_PARAM &T_SUB &T_SUPER &T_SPACE 
 		   &T_LETTER &T_OTHER &T_ACTIVE &T_COMMENT &T_CS
 		   &Token &Tokens
-		   &Tokenize &TokenizeInternal &Explode ),
+		   &Tokenize &TokenizeInternal &Explode &UnTeX),
 	       # Number & Dimension constructors
-	       qw( &Number &Dimension &MuDimension &Glue &MuGlue),
+	       qw( &Number &Float &Dimension &MuDimension &Glue &MuGlue &Pair &PairList),
 	       # Error & Progress reporting
-	       qw( &NoteProgress &Fatal &Error &Warn ),
+	       qw( &NoteProgress &NoteBegin &NoteEnd &Fatal &Error &Warn ),
 	       # And some generics
 	       qw(&Stringify &ToString  &Equals)
 );
@@ -78,10 +79,13 @@ sub Token {
 #======================================================================
 # These belong to Mouth, but make more sense here.
 
+# WARNING: These two utilities bind $STATE to simple State objects with known fixed catcodes.
+# The State normally contains ALL the bindings, etc and links to other important objects.
+# We CAN do that here, since we are ONLY tokenizing from a new Mouth, bypassing stomach & gullet.
+# However, be careful with any changes.
+
 our $STD_CATTABLE;
 our $STY_CATTABLE;
-# Note that we are overriding $STATE, which normally contains ALL the bindings, etc.
-# But, since we are just tokenizing, only the catcode bindings matter.
 
 # Tokenize($string); Tokenizes the string using the standard cattable, returning a LaTeXML::Tokens
 sub Tokenize         {
@@ -110,15 +114,21 @@ sub Explode {
   my($string)=@_;
   map(($_ eq ' ' ? T_SPACE() : T_OTHER($_)),split('',$string)); }
 
+sub UnTeX {
+  my($thing)=@_;
+  ToString(Tokens(ref $thing ? $thing->revert : Explode($thing))); }
+
 #======================================================================
 # Constructors for number and dimension types.
 
 sub Number      { LaTeXML::Number->new(@_); }
+sub Float       { LaTeXML::Float->new(@_); }
 sub Dimension   { LaTeXML::Dimension->new(@_); }
 sub MuDimension { LaTeXML::MuDimension->new(@_); }
 sub Glue        { LaTeXML::Glue->new(@_); }
 sub MuGlue      { LaTeXML::MuGlue->new(@_); }
-
+sub Pair        { LaTeXML::Pair->new(@_); }
+sub PairList    { LaTeXML::PairList->new(@_); }
 #**********************************************************************
 # Error & Progress reporting.
 
@@ -127,9 +137,23 @@ sub NoteProgress {
   print STDERR @_ if $LaTeXML::Global::STATE->lookupValue('VERBOSITY') >= 0;
   return; }
 
+our %note_timers=();
+sub NoteBegin {
+  my($state)=@_;
+  $note_timers{$state}=[Time::HiRes::gettimeofday];
+  print STDERR "\n($state..." if $LaTeXML::Global::STATE->lookupValue('VERBOSITY') >= 0; }
+
+sub NoteEnd {
+  my($state)=@_;
+  if(my $start = $note_timers{$state}){
+    my $elapsed = Time::HiRes::tv_interval($start,[Time::HiRes::gettimeofday]);
+    undef $note_timers{$state};
+    print STDERR sprintf(" %.2f sec)",$elapsed) if $LaTeXML::Global::STATE->lookupValue('VERBOSITY') >= 0; }}
+
 sub Fatal { 
   my($message)=@_;
   if(!$LaTeXML::Error::InHandler && defined($^S)){
+    $LaTeXML::Global::STATE->noteStatus('fatal');
     $message
       = LaTeXML::Error::generateMessage("Fatal",$message,1,
 		       ($LaTeXML::Global::STATE->lookupValue('VERBOSITY') > 0
@@ -139,18 +163,25 @@ sub Fatal {
   die $message; 
   return; }
 
+# Note that "100" is hardwired into TeX, The Program!!!
+our $MAXERRORS=100;
+
 # Should be fatal if strict is set, else warn.
 sub Error {
   my($msg)=@_;
   if($LaTeXML::Global::STATE->lookupValue('STRICT')){
     Fatal($msg); }
   else {
+    $LaTeXML::Global::STATE->noteStatus('error');
     print STDERR LaTeXML::Error::generateMessage("Error",$msg,1,"Continuing... Expect trouble.\n")
       unless $LaTeXML::Global::STATE->lookupValue('VERBOSITY') < -1; }
+  if(($LaTeXML::Global::STATE->getStatus('error')||0) > $MAXERRORS){
+    Fatal("Too many errors!"); }
   return; }
 
 sub Warn {
   my($msg)=@_;
+  $LaTeXML::Global::STATE->noteStatus('warning');
   print STDERR LaTeXML::Error::generateMessage("Warning",$msg,0)
     unless $LaTeXML::Global::STATE->lookupValue('VERBOSITY') < 0; 
   return; }
@@ -168,18 +199,23 @@ sub Stringify {
   # Have to handle LibXML stuff explicitly (unless we want to add methods...?)
   elsif($object->isa('XML::LibXML::Node')){
     if($object->nodeType == XML_ELEMENT_NODE){ 
-#      "$object"."[".$object->nodeName."]";
+      my $nodename = $object->localname;
+      my $ns    = $object->namespaceURI;
+      if($ns && ($ns=$LaTeXML::Global::STATE->getModel->getNamespacePrefix($ns))){
+	$nodename = "$ns:$nodename"; }
       my $attributes ='';
       foreach my $attr ($object->attributes){
 	my $name = $attr->nodeName;
 	next if $name =~ /^_/;
 	$attributes .= ' '. $name. "=\"".$attr->getData."\""; }
-      "<".$object->nodeName.$attributes.">";
+      "<".$nodename.$attributes. ($object->hasChildNodes ? ">..." : "/>");
     }
+    elsif($object->nodeType == XML_TEXT_NODE){
+      "XMLText[".$object->data."]"; }
+    elsif($object->nodeType == XML_DOCUMENT_NODE){
+      "XMLDocument[".$$object."]"; }
     else { "$object"; }}
   else { "$object"; }}
-#  (defined $object ? (((ref $object) && !$NOBLESS{ref $object}) && $object->can('stringify') ? $object->stringify : "$object")
-#   : 'undef'); }
 
 sub ToString {
   my($object)=@_;
@@ -200,7 +236,7 @@ __END__
 
 =head1 NAME
 
-C<LaTeXML::Global> -- global exports used within LaTeXML, and in Packages.
+C<LaTeXML::Global> - global exports used within LaTeXML, and in Packages.
 
 =head1 SYNOPSIS
 
@@ -215,10 +251,10 @@ throughout LaTeXML, and in Package implementations.
 
 =over 4
 
-=item C<< $STATE, $GULLET, $STOMACH, $MODEL; >>
+=item C<< $STATE; >>
 
-These are bound to the currently active L<LaTeXML::State>, L<LaTeXML::Gullet>, L<LaTeXML::Stomach>
-and L<LaTeXML::Model> by an instance of L<LaTeXML> during processing.
+This is bound to the currently active L<LaTeXML::State> by an instance
+of L<LaTeXML> during processing.
 
 =back 
 
@@ -228,18 +264,25 @@ and L<LaTeXML::Model> by an instance of L<LaTeXML> during processing.
 
 =item C<< $catcode = CC_ESCAPE; >>
 
-A constant for the escape category code; and also:
-C<CC_BEGIN>, C<CC_END>, C<CC_MATH>, C<CC_ALIGN>, C<CC_EOL>, C<CC_PARAM>, C<CC_SUPER>,
-C<CC_SUB>, C<CC_IGNORE>, C<CC_SPACE>, C<CC_LETTER>, C<CC_OTHER>, C<CC_ACTIVE>, C<CC_COMMENT>, 
-C<CC_INVALID>, C<CC_CS>, C<CC_NOTEXPANDED>.  [The last 2 are (apparent) extensions, 
+Constants for the category codes:
+
+  CC_BEGIN, CC_END, CC_MATH, CC_ALIGN, CC_EOL,
+  CC_PARAM, CC_SUPER, CC_SUB, CC_IGNORE,
+  CC_SPACE, CC_LETTER, CC_OTHER, CC_ACTIVE,
+  CC_COMMENT, CC_INVALID, CC_CS, CC_NOTEXPANDED.
+
+[The last 2 are (apparent) extensions,
 with catcodes 16 and 17, respectively].
 
 =item C<< $token = Token($string,$cc); >>
 
-Creates a L<LaTeXML::Token> with the given content and catcode.  The following shorthand versions
-are also exported for convenience:
-C<T_BEGIN>, C<T_END>, C<T_MATH>, C<T_ALIGN>, C<T_PARAM>, C<T_SUB>, C<T_SUPER>, C<T_SPACE>, 
-C<T_LETTER($letter)>, C<T_OTHER($char)>, C<T_ACTIVE($char)>, C<T_COMMENT($comment)>, C<T_CS($cs)>
+Creates a L<LaTeXML::Token> with the given content and catcode.
+The following shorthand versions are also exported for convenience:
+
+  T_BEGIN, T_END, T_MATH, T_ALIGN, T_PARAM,
+  T_SUB, T_SUPER, T_SPACE, T_LETTER($letter),
+  T_OTHER($char), T_ACTIVE($char),
+  T_COMMENT($comment), T_CS($cs)
 
 =item C<< $tokens = Tokens(@token); >>
 
@@ -268,6 +311,11 @@ Returns a list of the tokens corresponding to the characters in C<$string>.
 
 Creates a Number object representing C<$num>.
 
+=item C<< $number = Float($num); >>
+
+Creates a floating point object representing C<$num>;
+This is not part of TeX, but useful.
+
 =item C<< $dimension = Dimension($dim); >>
 
 Creates a Dimension object.  C<$num> can be a string with the number and units
@@ -294,6 +342,18 @@ or 1,2,3 for fil, fill or filll.
 
 Creates a MuGlue object, similar to Glue.
 
+
+=item C<< $pair = Pair($num1,$num2); >>
+
+Creates an object representing a pair of numbers;
+Not a part of TeX, but useful for graphical objects.
+The two components can be any numerical object.
+
+=item C<< $pair = PairList(@pairs); >>
+
+Creates an object representing a list of pairs of numbers;
+Not a part of TeX, but useful for graphical objects.
+
 =back
 
 =head2 Error Reporting
@@ -328,7 +388,7 @@ Prints C<$message> unless the verbosity level below 0.
 
 =item C<< Stringify($object); >>
 
-Returns a short string identifying C<$object>, for debugging purposes.
+Returns a short string identifying C<$object>, for debugging.
 Works on any values and objects, but invokes the stringify method on 
 blessed objects.
 More informative than the default perl conversion to a string.
