@@ -13,10 +13,8 @@
 package LaTeXML::Gullet;
 use strict;
 use LaTeXML::Global;
-use LaTeXML::Object;
 use LaTeXML::Util::Pathname;
-our @ISA = qw(LaTeXML::Object);
-
+use base qw(LaTeXML::Object);
 #**********************************************************************
 sub new {
   my($class)=@_;
@@ -44,28 +42,41 @@ sub input {
   # Try to find a Package implementing $name.
   local @LaTeXML::PACKAGE_OPTIONS = @{$options{options}||[]};
   $name = $1 if $name =~ /^\{(.*)\}$/; # just in case
-  my @paths = (@{ $STATE->lookupValue('SEARCHPATHS') },
-	       map("$_/LaTeXML/Package", @INC));
-  my $file = pathname_find($name,paths=>[@paths], types=>$types);
+  my $file = pathname_find($name,paths=>$STATE->lookupValue('SEARCHPATHS'),
+			   types=>$types, installation_subdir=>'Package');
   if(! $file) {
-    Error("Cannot find file $name of type ".join(', ',@{$types||[]})." in paths ".join(', ',@paths)); }
-  elsif($file =~ /\.(ltxml|latexml)$/){		# Perl module.
+    $STATE->noteStatus(missing=>$name);
+    Error("Cannot find file $name of type ".join(', ',@{$types||[]})
+	  ." in paths ".join(', ',@{$STATE->lookupValue('SEARCHPATHS')})); }
+#  elsif($file =~ /\.(ltxml|latexml)$/){		# Perl module.
+  elsif($file =~ /\.ltxml$/){		# Perl module.
     return if $STATE->lookupValue($file.'_loaded');
     $STATE->assignValue($file.'_loaded'=>1,'global');
-    my($dir,$modname)=pathname_split($file);
     $self->openMouth(LaTeXML::PerlMouth->new($file),0);
+    my $pmouth = $$self{mouth};
     do $file; 
-    Fatal("Package $name had an error:\n  $@") if $@; }
-  elsif($file =~ /\.sty$/){	# (attempt to) interpret a style file.
+    Fatal("Package $name had an error:\n  $@") if $@; 
+    $self->closeMouth if $pmouth eq $$self{mouth}; # Close immediately, unless recursive input
+  }
+  elsif($file =~ /\.(sty|cls)$/){	# (attempt to) interpret a style file.
     return if $STATE->lookupValue($file.'_loaded');
     $STATE->assignValue($file.'_loaded'=>1,'global');
     $self->openMouth(LaTeXML::StyleMouth->new($file), 0);  }
   else {			# Else read as an included file.
-    $self->openMouth(LaTeXML::FileMouth->new($file) ,0);  
-    # AND if there are file-specific declarations, deal with those first!
-    $file =~ s/\.tex//;
-    if(my $conf = pathname_find("$file.latexml",paths=>$STATE->lookupValue('SEARCHPATHS'))){
-      $self->input($conf); }
+    # If there is a file-specific declaration file (name.latexml), load it first!
+    my $name = $file;
+    $name =~ s/\.tex//;
+    local $LaTeXML::INHIBIT_LOAD=0;
+    if(my $conf = pathname_find("$name.latexml",
+				paths=>$STATE->lookupValue('SEARCHPATHS'))){
+      $self->openMouth(LaTeXML::PerlMouth->new($conf),0);
+      my $pmouth = $$self{mouth};
+      do $conf; 
+      Fatal("Configuration file $conf had an error:\n  $@") if $@; 
+      $self->closeMouth if $pmouth eq $$self{mouth}; # Close immediately, unless rec. input
+    }
+    # NOW load the input --- UNLESS INHIBITTED!!!
+    $self->openMouth(LaTeXML::FileMouth->new($file) ,0) unless $LaTeXML::INHIBIT_LOAD;
   }}
 
 #**********************************************************************
@@ -115,12 +126,13 @@ sub getLocator {
   my $loc = (defined $$self{mouth} ? $$self{mouth}->getLocator($long) : '');
   if(!$loc || $long){
     my($mouth,$pb)=($$self{mouth},$$self{pushback});
-    $loc .= "\n  To be read again ".Tokens(@$pb)->untex if $long && @$pb;
+    $loc .= "\n  To be read again ".ToString(Tokens(@$pb)) if $long && @$pb;
     foreach my $frame ( @{$$self{mouthstack}} ){
       my($mouth,$pb)= @$frame;
-      $loc .= ' '.$mouth->getLocator($long);
+      my $ml = $mouth->getLocator($long);
+      $loc .= ' '.$ml if $ml;
       last if $loc && !$long;
-      $loc .= "\n  To be read again ".Tokens(@$pb)->untex if $long && @$pb;
+      $loc .= "\n  To be read again ".ToString(Tokens(@$pb)) if $long && @$pb;
     }}
   $loc; }
 
@@ -128,7 +140,7 @@ sub getLocator {
 # Return $tokens with all tokens expanded
 sub expandTokens {
   my($self,$tokens)=@_;
-  $self->openMouth($tokens->clone, 1);
+  $self->openMouth((ref $tokens eq 'LaTeXML::Token' ? Tokens($tokens) : $tokens->clone),1);
   my @expanded=();
   while(defined(my $t=$self->readXToken)){
     push(@expanded,$t);}
@@ -142,9 +154,9 @@ sub neutralizeTokens {
   my($self,@tokens)=@_;
   my @result=();
   foreach my $t (@tokens){
-    if($t->getCatcode == CC_PARAM){    
+    if($t->getCatcode == CC_PARAM){
       push(@result,$t); }
-    elsif(defined(my $defn=$t->getDefinition)){
+    elsif(defined(my $defn=$STATE->lookupDefinition($t))){
       push(@result,Token('\noexpand',CC_NOTEXPANDED)); }
     push(@result,$t); }
   @result; }
@@ -191,8 +203,8 @@ sub readXToken {
     elsif($cc == CC_COMMENT){
       return $token if $toplevel;
       push(@{$$self{pending_comments}},$token); } # What to do with comments???
-    elsif(defined($defn=$token->getDefinition) && $defn->isExpandable){
-      $self->unread($defn->invoke); } # Expand and push back the result (if any) and continue
+    elsif(defined($defn=$STATE->lookupDefinition($token)) && $defn->isExpandable){
+      $self->unread($defn->invoke($self)); } # Expand and push back the result (if any) and continue
     else {
       return $token; }		# just return it
   }}
@@ -243,14 +255,11 @@ sub readBalanced {
   Tokens(@tokens); }
 
 sub ifNext {
-  my($self,$string,$cc)=@_;
-  my $tok=$self->readToken();
-  if(defined $string && ref $string eq 'LaTeXML::Token'){
-    $cc=$string->getCatcode; $string=$string->getString; }
-  return $tok if (defined $tok && ((!defined $string) || ($string eq $tok->getString))
-		  && ((!defined $cc) || ($cc == $tok->getCatcode)) );
-  $self->unread($tok) if defined $tok;
-  0; }
+  my($self,$token)=@_;
+  if(my $tok=$self->readToken()){
+    $self->unread($tok);
+    $tok->equals($token); }
+  else { 0; }}
 
 # Match the input against one of the Token or Tokens in @choices; return the matching one or undef.
 sub readMatch {
@@ -259,8 +268,14 @@ sub readMatch {
     my @tomatch=$choice->unlist;
     my @matched=();
     my $tok;
-    while(@tomatch && defined($tok=$self->readToken) && push(@matched,$tok) && ($tok->equals($tomatch[0]))){
-      shift(@tomatch); }
+    while(@tomatch && defined($tok=$self->readToken)
+	  && push(@matched,$tok) && ($tok->equals($tomatch[0]))){
+      shift(@tomatch);
+      if($tok->getCatcode == CC_SPACE){ # If this was space, SKIP any following!!!
+	while(defined($tok=$self->readToken) && ($tok->getCatcode == CC_SPACE)){
+	  push(@matched,$tok); }
+	$self->unread($tok) if $tok; }
+    }
     return $choice unless @tomatch;	# All matched!!!
     $self->unread(@matched);	# Put 'em back and try next!
   }
@@ -272,6 +287,7 @@ sub readKeyword {
   my($self,@keywords)=@_;
   $self->skipSpaces;
   foreach my $keyword (@keywords){
+    $keyword = $keyword->toString if ref $keyword;
     my @tomatch=split('',uc($keyword));
     my @matched=();
     my $tok;
@@ -322,24 +338,6 @@ sub readOptional {
     $self->unread($tok);
     $default; }}
 
-# Like readarg, but with catcodes changed to a semi-verbatim form,
-# such as for url's and such.
-sub readSemiverbatim {
-  my($self)=@_;
-  $self->startSemiverbatim;
-  my $arg = $self->readArg;
-  $self->endSemiverbatim;
-  $arg; }
-
-sub startSemiverbatim {
-  my($self)=@_;
-  $STATE->pushFrame;
-  map($STATE->assignCatcode($_=>CC_OTHER,'local'),'^','_','@','~','&','$','#','%');  # should '%' too ?
-}
-sub endSemiverbatim {
-  my($self)=@_;
-  $STATE->popFrame; }
-
 #**********************************************************************
 #  Numbers, Dimensions, Glue
 # See TeXBook, Ch.24, pp.269-271.
@@ -357,7 +355,7 @@ sub readRegisterValue {
   my($self,$type)=@_;
   my $token = $self->readXToken;
   return unless defined $token;
-  my $defn = $token->getDefinition;
+  my $defn = $STATE->lookupDefinition($token);
   if((defined $defn) && ($defn->isRegister eq $type)){
     $defn->valueOf($defn->readArguments($self)); }
   else {
@@ -442,6 +440,29 @@ sub readNormalInteger {
     $self->readInternalInteger; }}
 
 sub readInternalInteger{ $_[0]->readRegisterValue('Number'); }
+
+#======================================================================
+# Float, a floating point number.
+# Similar to factor, but does NOT accept comma!
+# This is NOT part of TeX, but is convenient.
+sub readFloat {
+  my($self)=@_;
+  my $s = $self->readOptionalSigns;
+  my $string = $self->readDigits('0-9');
+  my $token = $self->readXToken;
+  if($token && $token->getString =~ /^[\.]$/){
+    $string .= '.'.$self->readDigits('0-9'); 
+    $token = $self->readXToken; }
+  my $n;
+  if(length($string)>0){
+    $self->unread($token) if $token && $token->getCatcode!=CC_SPACE;
+    $n = $string; }
+  else {
+    $self->unread($token);
+    $n = $self->readNormalInteger;
+    $n = $n->valueOf if defined $n; }
+  (defined $n ? Float($s*$n) : undef); }
+
 #======================================================================
 # Dimensions
 #======================================================================
@@ -453,7 +474,12 @@ sub readDimension {
   my $s = $self->readOptionalSigns;
   if   (defined (my $d = $self->readInternalDimension)){ ($s < 0 ? $d->negate : $d); }
   elsif(defined (   $d = $self->readInternalGlue)     ){ Dimension($s * $d->valueOf); }
-  elsif(defined (   $d = $self->readFactor)           ){ Dimension($s * $d * $self->readUnit); }
+  elsif(defined (   $d = $self->readFactor)           ){ 
+    my $unit = $self->readUnit;
+    if(!defined $unit){
+      Warn("Illegal unit of measure (pt inserted).");
+      $unit = 65536; }
+    Dimension($s * $d * $unit); }
   else{ Warn("Missing number, treated as zero.");        Dimension(0); }}
 
 # <unit of measure> = <optional spaces><internal unit>
@@ -473,7 +499,7 @@ sub readUnit {
     $self->readKeyword('true');	# But ignore, we're not bothering with mag...
     $u = $self->readKeyword('pt','pc','in','bp','cm','mm','dd','cc','sp');
     if($u){ $self->skip1Space; $STATE->convertUnit($u); }
-    else  { Warn("Illegal unit of measure (pt inserted)."); 65536; }}}
+    else  { undef; }}}
 
 # Return a dimension value or undef
 sub readInternalDimension { $_[0]->readRegisterValue('Dimension'); }
@@ -489,7 +515,12 @@ sub readInternalDimension { $_[0]->readRegisterValue('Dimension'); }
 sub readMuDimension {
   my($self)=@_;
   my $s = $self->readOptionalSigns;
-  if   (defined (my $m = $self->readFactor        )){ MuDimension($s * $m * $self->readMuUnit); }
+  if   (defined (my $m = $self->readFactor        )){
+    my $munit = $self->readMuUnit;
+    if(!defined $munit){
+      Warn("Illegal unit of measure (mu inserted).");
+      $munit = $STATE->convertUnit('mu'); }
+    MuDimension($s * $m * $munit); }
   elsif(defined (   $m = $self->readInternalMuGlue)){ MuDimension($s * $m->valueOf); }
   else{ Warn("Expecting mudimen; assuming 0 ");       MuDimension(0); }}
 
@@ -497,7 +528,7 @@ sub readMuUnit {
   my($self)=@_;
   if   (my $m=$self->readKeyword('mu')){ $self->skip1Space; $STATE->convertUnit($m); }
   elsif($m=$self->readInternalMuGlue  ){ $m->valueOf; }
-  else { Warn("Illegal unit of measure (mu inserted)."); $STATE->convertUnit('mu'); }}
+  else { undef; }}
 
 #======================================================================
 # Glue
@@ -531,11 +562,18 @@ sub readRubber {
     ($f->valueOf * $s, 0); }
   elsif(defined(my $fil = $self->readKeyword('filll','fill','fil'))){
     ($s*$f,$FILLS{$fil}); }
-  elsif(defined(my $u = ($mu ? $self->readMuUnit : $self->readUnit))){
+  elsif($mu){
+    my $u = $self->readMuUnit;
+    if(!defined $u){
+      Warn("Illegal unit of measure (mu inserted).");
+      $u = $STATE->convertUnit('mu'); }
     ($s*$f*$u,0); }
   else {
-    Warn("Illegal unit of measure (pt inserted).");
-    ($s*$f*65536,0); }}
+    my $u = $self->readUnit;
+    if(!defined $u){
+      Warn("Illegal unit of measure (pt inserted).");
+      $u = 65536; }
+    ($s*$f*$u,0); }}
 
 # Return a glue value or undef.
 sub readInternalGlue { $_[0]->readRegisterValue('Glue'); }
@@ -579,7 +617,7 @@ __END__
 
 =head1 NAME
 
-C<LaTeXML::Gullet> -- expands expandable tokens and parses common token sequences.
+C<LaTeXML::Gullet> - expands expandable tokens and parses common token sequences.
 
 =head1 DESCRIPTION
 
@@ -595,7 +633,7 @@ to TeX's rules.
 
 =over 4
 
-=item C<< $GULLET->input($name,$types,%options); >>
+=item C<< $gullet->input($name,$types,%options); >>
 
 Input the file named C<$name>; Searches for matching files in the
 current C<searchpath> with an extension being one of  C<$types> (an array
@@ -604,22 +642,22 @@ it will be executed (loaded).  If the found file has a TeX extension
 (tex, sty, cls) it will be opened and latexml will prepare to read
 from it.
 
-=item C<< $GULLET->openMouth($mouth, $noautoclose); >>
+=item C<< $gullet->openMouth($mouth, $noautoclose); >>
 
 Is this public? Prepares to read tokens from C<$mouth>.
 If $noautoclose is true, the Mouth will not be automatically closed
 when it is exhausted.
 
-=item C<< $GULLET->closeMouth; >>
+=item C<< $gullet->closeMouth; >>
 
 Is this public? Finishes reading from the current mouth, and
 reverts to the one in effect before the last openMouth.
 
-=item C<< $GULLET->flush; >>
+=item C<< $gullet->flush; >>
 
 Is this public? Clears all inputs.
 
-=item C<< $GULLET->getLocator; >>
+=item C<< $gullet->getLocator; >>
 
 Returns a string describing the current location in the input stream.
 
@@ -629,27 +667,27 @@ Returns a string describing the current location in the input stream.
 
 =over 4
 
-=item C<< $tokens = $GULLET->expandTokens($tokens); >>
+=item C<< $tokens = $gullet->expandTokens($tokens); >>
 
-Return a L<LaTeXML::Tokens> being the expansion of all the tokens in C<$tokens>.
+Return the L<LaTeXML::Tokens> resulting from expanding all the tokens in C<$tokens>.
 This is actually only used in a few circumstances where the arguments to
 an expandable need explicit expansion; usually expansion happens at the right time.
 
-=item C<< @tokens = $GULLET->neutralizeTokens(@tokens); >>
+=item C<< @tokens = $gullet->neutralizeTokens(@tokens); >>
 
 Another unusual method: Used for things like \edef and token registers, to
 inhibit further expansion of control sequences and proper spawning of register tokens.
 
-=item C<< $token = $GULLET->readToken; >>
+=item C<< $token = $gullet->readToken; >>
 
 Return the next token from the input source, or undef if there is no more input.
 
-=item C<< $token = $GULLET->readXToken($toplevel); >>
+=item C<< $token = $gullet->readXToken($toplevel); >>
 
 Return the next unexpandable token from the input source, or undef if there is no more input.
 If the next token is expandable, it is expanded, and its expansion is reinserted into the input.
 
-=item C<< $GULLET->unread(@tokens); >>
+=item C<< $gullet->unread(@tokens); >>
 
 Push the C<@tokens> back into the input stream to be re-read.
 
@@ -659,40 +697,40 @@ Push the C<@tokens> back into the input stream to be re-read.
 
 =over 4
 
-=item C<< $token = $GULLET->readNonSpace; >>
+=item C<< $token = $gullet->readNonSpace; >>
 
-Read and return the next non-space token from the input, discarding any spaces.
+Read and return the next non-space token from the input after discarding any spaces.
 
-=item C<< $GULLET->skipSpaces; >>
+=item C<< $gullet->skipSpaces; >>
 
 Skip the next spaces from the input.
 
-=item C<< $GULLET->skip1Space; >>
+=item C<< $gullet->skip1Space; >>
 
 Skip the next token from the input if it is a space.
 
-=item C<< $tokens = $GULLET->readBalanced; >>
+=item C<< $tokens = $gullet->readBalanced; >>
 
 Read a sequence of tokens from the input until the balancing '}' (assuming the '{' has
 already been read). Returns a L<LaTeXML::Tokens>.
 
-=item C<< $boole = $GULLET->ifNext($string,$catcode); >>
+=item C<< $boole = $gullet->ifNext($token); >>
 
-Returns true if the next token in the input matches the C<$string> and/or C<$catcode>.
-(either can be undef, or C<$string> can be a token).
+Returns true if the next token in the input matches C<$token>;
+the possibly matching token remains in the input.
 
-=item C<< $tokens = $GULLET->readMatch(@choices); >>
+=item C<< $tokens = $gullet->readMatch(@choices); >>
 
-Read and return whichever of C<@choices> (each should be a L<LaTeXML::Tokens>)
+Read and return whichever of C<@choices> (each are L<LaTeXML::Tokens>)
 matches the input, or undef if none do.
 
-=item C<< $keyword = $GULLET->readKeyword(@keywords); >>
+=item C<< $keyword = $gullet->readKeyword(@keywords); >>
 
-Read and return whichever of C<@keywords> (each should be a string) matches the input, or undef
+Read and return whichever of C<@keywords> (each a string) matches the input, or undef
 if none do.  This is similar to readMatch, but case and catcodes are ignored.
 Also, leading spaces are skipped.
 
-=item C<< $tokens = $GULLET->readUntil(@delims); >>
+=item C<< $tokens = $gullet->readUntil(@delims); >>
 
 Read and return a (balanced) sequence of L<LaTeXML::Tokens> until  matching one of the tokens
 in C<@delims>.  In a list context, it also returns which of the delimiters ended the sequence.
@@ -703,50 +741,45 @@ in C<@delims>.  In a list context, it also returns which of the delimiters ended
 
 =over 4
 
-=item C<< $tokens = $GULLET->readArg; >>
+=item C<< $tokens = $gullet->readArg; >>
 
 Read and return a TeX argument; the next Token or Tokens (if surrounded by braces).
 
-=item C<< $tokens = $GULLET->readOptional($default); >>
+=item C<< $tokens = $gullet->readOptional($default); >>
 
 Read and return a LaTeX optional argument; returns C<$default> if there is no '[',
 otherwise the contents of the [].
 
-=item C<< $tokens = $GULLET->readSemiverbatim; >>
-
-Read and return a TeX argument, but with catcodes reset so that most annoying
-characters are treated as OTHER; useful for reading pathnames, URL's, etc.
-
-=item C<< $thing = $GULLET->readValue($type); >>
+=item C<< $thing = $gullet->readValue($type); >>
 
 Reads an argument of a given type: one of 'Number', 'Dimension', 'Glue', 'MuGlue' or 'any'.
 
-=item C<< $value = $GULLET->readRegisterValue($type); >>
+=item C<< $value = $gullet->readRegisterValue($type); >>
 
 Read a control sequence token (and possibly it's arguments) that names a register,
 and return the value.  Returns undef if the next token isn't such a register.
 
-=item C<< $number = $GULLET->readNumber; >>
+=item C<< $number = $gullet->readNumber; >>
 
 Read a L<LaTeXML::Number> according to TeX's rules of the various things that
 can be used as a numerical value. 
 
-=item C<< $dimension = $GULLET->readDimension; >>
+=item C<< $dimension = $gullet->readDimension; >>
 
 Read a L<LaTeXML::Dimension> according to TeX's rules of the various things that
 can be used as a dimension value.
 
-=item C<< $mudimension = $GULLET->readMuDimension; >>
+=item C<< $mudimension = $gullet->readMuDimension; >>
 
 Read a L<LaTeXML::MuDimension> according to TeX's rules of the various things that
 can be used as a mudimension value.
 
-=item C<< $glue = $GULLET->readGlue; >>
+=item C<< $glue = $gullet->readGlue; >>
 
 Read a  L<LaTeXML::Glue> according to TeX's rules of the various things that
 can be used as a glue value.
 
-=item C<< $muglue = $GULLET->readMuGlue; >>
+=item C<< $muglue = $gullet->readMuGlue; >>
 
 Read a L<LaTeXML::MuGlue> according to TeX's rules of the various things that
 can be used as a muglue value.
