@@ -17,8 +17,7 @@
 package LaTeXML::Mouth;
 use strict;
 use LaTeXML::Global;
-use LaTeXML::Object;
-our @ISA = qw(LaTeXML::Object);
+use base qw(LaTeXML::Object);
 
 sub new {
   my($class,$string)=@_;
@@ -228,7 +227,7 @@ sub readRawLines {
       last; }
     elsif(!$exact && ($line =~ /^(.*?)\Q$endline\E(.*)$/)){
       my($pre,$post)=($1,$2);
-      push(@lines,$pre) if $pre;
+      push(@lines,$pre."\n") if $pre;
       $$self{chars}=[split('',$line)];
       $$self{nchars} = scalar(@{$$self{chars}});
       $$self{colno} = length($pre)+length($endline);
@@ -244,16 +243,18 @@ sub readRawLines {
 package LaTeXML::FileMouth;
 use strict;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::Mouth);
+use LaTeXML::Util::Pathname;
+use base qw(LaTeXML::Mouth);
 
 sub new {
   my($class,$pathname)=@_;
   local *IN;
   open(IN,$pathname) || Fatal("Can't read from $pathname");
-  my $self = {pathname=>$pathname, source=>$pathname, IN => *IN};
+  my $shortpath=pathname_relative($pathname,pathname_cwd);
+  my $self = {pathname=>$pathname, source=>$shortpath, IN => *IN};
   bless $self,$class;
   $self->initialize;
-  NoteProgress("\n(Processing $pathname");
+  NoteBegin("Processing $$self{source}");
   $self;  }
 
 sub finish {
@@ -261,7 +262,7 @@ sub finish {
   $self->SUPER::finish;
   if($$self{IN}){
     close( \*{$$self{IN}}); $$self{IN}=undef; }
-  NoteProgress(")");}
+  NoteEnd("Processing $$self{source}");}
 
 sub hasMoreInput {
   my($self)=@_;
@@ -288,16 +289,18 @@ sub stringify {
 package LaTeXML::StyleMouth;
 use strict;
 use LaTeXML::Global;
-our @ISA = qw(LaTeXML::FileMouth);
+use LaTeXML::Util::Pathname;
+use base qw(LaTeXML::FileMouth);
 
 sub new {
   my($class,$pathname)=@_;
   local *IN;
   open(IN,$pathname) || Fatal("Can't read from $pathname");
-  my $self = {pathname=>$pathname, source=>$pathname, IN => *IN};
+  my $shortpath=pathname_relative($pathname,pathname_cwd);
+  my $self = {pathname=>$pathname, source=>$shortpath, IN => *IN};
   bless $self,$class;
   $self->initialize;
-  NoteProgress("\n(Style $pathname");
+  NoteBegin("Style $$self{source}");
   $$self{saved_at_cc} = $STATE->lookupCatcode('@');
   $$self{SAVED_INCLUDE_COMMENTS} = $STATE->lookupValue('INCLUDE_COMMENTS');
   $STATE->assignCatcode('@'=>CC_LETTER);
@@ -308,6 +311,7 @@ sub finish {
   my($self)=@_;
   $STATE->assignCatcode('@'=> $$self{saved_at_cc});
   $STATE->assignValue(INCLUDE_COMMENTS=>$$self{SAVED_INCLUDE_COMMENTS});
+  NoteEnd("Style $$self{source}");
   $self->SUPER::finish; }
 
 #**********************************************************************
@@ -316,22 +320,25 @@ sub finish {
 package LaTeXML::PerlMouth;
 use strict;
 use LaTeXML::Global;
+use LaTeXML::Util::Pathname;
 
 sub new {
-  my($class,$file)=@_;
-  my $self = bless {file=>$file},$class;
-  NoteProgress("\n(Loading $file");
+  my($class,$pathname)=@_;
+  my $shortpath=pathname_relative($pathname,pathname_cwd);
+  my $self = bless {pathname=>$pathname, source=>$shortpath},$class;
+  NoteBegin("Loading $$self{source}");
   $self; }
 
 sub finish {
-  NoteProgress(")"); }
+  my($self)=@_;
+  NoteEnd("Loading $$self{source}"); }
 
 # Evolve to figure out if this gets dynamic location!
 sub getLocator {
   my($self)=@_;
-  my $file = $$self{file};
-  my $line = LaTeXML::Error::line_in_file($file);
-  $file.($line ? " line $line":''); }
+  my $path = $$self{pathname};
+  my $line = LaTeXML::Error::line_in_file($path);
+  $path.($line ? " line $line":''); }
 
 sub hasMoreInput { 0; }
 sub readToken { undef; }
@@ -346,14 +353,15 @@ __END__
 
 =head1 NAME
 
-C<LaTeXML::Mouth>, C<LaTeXML::FileMouth> and C<LaTeXML::StyleMouth> -- tokenize
-the input.
+C<LaTeXML::Mouth> - tokenize the input.
 
 =head1 DESCRIPTION
 
 A C<LaTeXML::Mouth> (and subclasses) is responsible for I<tokenizing>, ie.
 converting plain text and strings into L<LaTeXML::Token>s according to the
 current category codes (catcodes) stored in the C<LaTeXML::State>.
+
+
 C<LaTeXML::FileMouth> specializes C<LaTeXML::Mouth> to tokenize from a file.
 C<LaTeXML::StyleMouth> further specializes C<LaTeXML::FileMouth> for processing
 style files, setting the catcode for C<@> and ignoring comments.
@@ -402,9 +410,10 @@ This is useful for the C<\verb> command.
 
 =item C<< $lines = $mouth->readRawLines($endline,$exact); >>
 
-Reads raw lines (not tokenized) from C<$mouth> until a line matches C<$endline>.
-If C<$exact> is true, the matching is done like with the c<comment> package;
-the ending line must match exactly, with no leading or trailing data.
+Reads raw (untokenized) lines from C<$mouth> until a line matching C<$endline>
+is found.
+If C<$exact> is true, C<$endline> is matched exactly, with no leading or trailing
+data (like in the c<comment> package).
 Otherwise, the match is done like with the c<verbatim> environment;
 any text preceding C<$endline> is returned as the last line, and any characters
 after C<$endline> remains in the mouth to be tokenized.
