@@ -12,29 +12,16 @@
 package LaTeXML::Post::PurgeXMath;
 use strict;
 use LaTeXML::Util::LibXML;
-use Exporter;
-use LaTeXML::Post;
-our @ISA = (qw(Exporter LaTeXML::Post::Processor));
+use base qw(LaTeXML::Post);
 
 # ================================================================================
-sub new {
-  my($class,%options)=@_;
-  my $self = bless {},$class;
-  $self->init(%options);
-  $self; }
-
 sub process {
-  my($self,$doc,%options)=@_;
-
-  my @math =  $self->findMathNodes($doc);
-  $self->Progress("Removing ".scalar(@math)." Intermediate XMath nodes");
-  foreach my $math (@math){
-    $math->parentNode->removeChild($math); }
-  $doc; }
-
-sub findMathNodes {
   my($self,$doc)=@_;
-  $doc->getElementsByTagNameNS($self->getNamespace,'XMath'); }
+  if(my @math = $doc->findnodes('//ltx:XMath')){
+    $self->Progress("Removing ".scalar(@math)." Intermediate XMath nodes");
+    foreach my $math (@math){
+      $math->parentNode->removeChild($math); }}
+  $doc; }
 
 # ================================================================================
 1;
