@@ -28,12 +28,14 @@
 package LaTeXML::Util::Pathname;
 use strict;
 use File::Spec;
+use File::Copy;
 use Cwd;
 use Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT = qw( &pathname_find
+our @EXPORT = qw( &pathname_find &pathname_findall
 		  &pathname_make &pathname_canonical
-		  &pathname_split &pathname_concat
+		  &pathname_split &pathname_directory &pathname_name &pathname_type
+		  &pathname_concat
 		  &pathname_relative &pathname_absolute
 		  &pathname_is_absolute 
 		  &pathname_cwd &pathname_mkdir &pathname_copy);
@@ -77,21 +79,34 @@ sub pathname_canonical {
 #  File::Spec->canonpath($pathname); }
   $pathname =~ s|//+|/|g;
   $pathname =~ s|/\./|/|g;
-  $pathname =~ s|/[^/]+/\.\./|/|g;
+  while($pathname =~ s|/[^/]+/\.\./|/|){}
   $pathname =~ s|^\./||;
   $pathname; }
 
+# Convenient extractors;
+sub pathname_directory { 
+  my($dir,$name,$type)=pathname_split(@_);
+  $dir; }
+
+sub pathname_name { 
+  my($dir,$name,$type)=pathname_split(@_);
+  $name; }
+
+sub pathname_type { 
+  my($dir,$name,$type)=pathname_split(@_);
+  $type; }
+
 #======================================================================
 sub pathname_concat {
   my($dir,$file)=@_;
-  File::Spec->catpath('',$dir,$file); }
+  File::Spec->catpath('',$dir || '',$file); }
 
 #======================================================================
 # Is $pathname an absolute pathname ?
 # pathname_is_absolute($pathname) => (0|1)
 sub pathname_is_absolute {
   my($pathname)=@_;
-  File::Spec->file_name_is_absolute($pathname); }
+  $pathname && File::Spec->file_name_is_absolute($pathname); }
 
 # pathname_relative($pathname,$base) => $relativepathname
 # Return $pathname as a pathname relative to $base.
@@ -109,6 +124,7 @@ sub pathname_cwd { cwd(); }
 
 sub pathname_mkdir {
   my($directory)=@_;
+  return undef unless $directory;
   my($volume,$dirs,$last)=File::Spec->splitpath($directory);
   my(@dirs)=(File::Spec->splitdir($dirs),$last);
   for(my $i=0; $i <= $#dirs; $i++){
@@ -122,46 +138,90 @@ sub pathname_mkdir {
 sub pathname_copy {
   my($source,$destination)=@_;
   # If it _needs_ to be copied:
-  if(!(-f $destination) || (-M $source < -M $destination)){
-    if($^O =~ /^(MSWin32|NetWare)$/){ # Windows
-      # According to Ioan, this should work:
-      system("xcopy /P $source $destination")==0 or return undef; }
-    else {			# Unix
-      system("cp -p $source $destination")==0 or return undef; }
-    # It would make sense to punt to File::Copy in the
-    # case where we are NOT unix or windows...
-    # But no clear set of $^O characterizes Unix!
+  if((!-f $destination) || (-M $source < -M $destination)){
+    pathname_mkdir(pathname_directory($destination)) or return undef;
+###    if($^O =~ /^(MSWin32|NetWare)$/){ # Windows
+###      # According to Ioan, this should work:
+###      system("xcopy /P $source $destination")==0 or return undef; }
+###    else {			# Unix
+###      system("cp --preserve=timestamps $source $destination")==0 or return undef; }
+    # Hopefully this portably copies, preserving timestamp.
+    copy($source,$destination) or return undef; 
+    my($atime,$mtime)= (stat($source))[8,9];
+    utime $atime,$mtime,$destination; # And set the modification time
   }
   return $destination; }
 
 #======================================================================
 # pathname_find($pathname, paths=>[...], types=>[...])  => $absolute_pathname;
-# Find a file corresponding to $pathname returning the absolute, completed pathname if found, else undef
-#  * If $pathname is a not an absolute pathname (it may still have directory components)
-#    then if search $paths are given, search for it relative to each of the directories in $paths,
+# Find a file corresponding to $pathname returning the absolute,
+# completed pathname if found, else undef
+#  * If $pathname is a not an absolute pathname 
+#    (although it may still have directory components)
+#    then if search $paths are given, search for it relative to
+#    each of the directories in $paths,
 #    else search for it relative to the current working directory.
-#  * If a pathname is not found, and 'types' is given, search for the pathname with 
-#    each type (extension) added.
-#    Note that if 'types' is given, it may find the file name w/o any extension.
-#    If a specific extension is required, use pathname_find(name="foo.type")
+#  * If types is given, then search (in each searched directory)
+#    for the first file with the given extension. 
+#    The extension "" (empty string) means to search for the exact name.
+#  * If types is not given, search for the exact named file
+#    without additional extension.
+#  * If installation_subdir is given, look in that subdirectory of where LaTeXML
+#    was installed, by appending it to the paths.
+
+our @INSTALLDIRS = grep(-d $_, map("$_/LaTeXML", @INC));
+
 sub pathname_find {
   my($pathname,%options)=@_;
-  my $paths = $options{paths};
-  my $types = $options{types};
-  if(pathname_is_absolute($pathname)){
-    if(-f $pathname){ return $pathname; }
-    elsif($types){
-      foreach my $type (@$types){
-	my $fullpath = $pathname . "." . $type;
-	return pathname_canonical($fullpath) if -f $fullpath; }}
-    return undef; }
-  elsif($paths){
-    foreach my $path (@$paths){
-      # make sure each $path is absolute ?
-      my $fullpath = pathname_find(pathname_concat($path,$pathname),types=>$types);
-      return $fullpath if $fullpath; }}
-  else {
-    pathname_find(pathname_concat(pathname_cwd(),$pathname),types=>$types); }}
+  return undef unless $pathname;
+  my @paths = candidate_pathnames($pathname,%options);
+  foreach my $path (@paths){
+    return $path if -f $path; }}
+
+sub pathname_findall {
+  my($pathname,%options)=@_;
+  return undef unless $pathname;
+  my @paths = candidate_pathnames($pathname,%options);
+  grep(-f $_, @paths); }
+
+# It's presumably cheep to concatinate all the pathnames,
+# relative to the cost of testing for files,
+# and this simplifies overall.
+sub candidate_pathnames {
+  my($pathname,%options)=@_;
+  my @dirs=('');
+  if(!pathname_is_absolute($pathname)){
+    my $cwd = pathname_cwd();
+    # Complete the search paths by prepending current dir to relative paths,
+    # but have at least the current dir.
+    @dirs = ($options{paths}
+	     ? map( (pathname_is_absolute($_) ? $_ : pathname_concat($cwd,$_)),
+		    @{$options{paths}})
+	     : ($cwd));
+    # And, if installation dir specified, append it.
+    if(my $subdir = $options{installation_subdir}){
+      push(@dirs,map(pathname_concat($_,$subdir),@INSTALLDIRS)); }}
+
+  # extract the desired extensions.
+  my @exts = ();
+  if($options{types}){
+    foreach my $ext (@{$options{types}}){
+      if($ext eq ''){ push(@exts,''); }
+      elsif($pathname =~ /\.\Q$ext\E$/i){
+	push(@exts,''); }
+      else {
+	# Half attempt at case insensitivity; not actually correct, though.
+	push(@exts,'.'.lc($ext)) if $ext =~/[A-Z]/;
+	push(@exts,'.'.uc($ext)) if $ext =~/[a-z]/;
+	push(@exts, '.'.$ext); }}}
+    push(@exts,'') unless @exts;
+
+  my @paths = ();
+  # Now, combine; precedence to leading directories.
+  foreach my $dir (@dirs){
+    foreach my $ext (@exts){
+      push(@paths,pathname_concat($dir,$pathname.$ext)); }}
+  @paths; }
 
 #======================================================================
 1;
